#version 330

in vec3 worldNormal;
in vec2 fragmentUV;
in vec3 surfacePosition;
in vec3 surfaceNormal;
in vec4 shadow_tex_coord;

out vec4 fragColor;

uniform sampler2D shadow_map; // shadow map is just a texture
uniform sampler2D colorTexture;
uniform float lightPower;
uniform float specExp;
uniform vec3 Ld;
uniform vec3 Ls;
uniform vec3 lightPosition;
uniform vec3 cameraPosition;

vec3 tex_coords;

float chebyshevUpperBound( float distance)
{
	vec2 moments = texture2D(shadow_map,tex_coords.xy).rg;
	
	// surface is fully lit, no occlusion
	if (distance <= moments.x)
		return 1.0 ;

	// fragment (partially) in shadow chebyshev's upperBound: how likely pixel is lit (p_max)
	float variance = moments.y - (moments.x*moments.x);
	variance = max(variance,0.00002);

	float d = distance - moments.x;
	float p_max = variance / (variance + d*d);

	return p_max;
}

void main()
{
	// perform perspective division
	tex_coords = shadow_tex_coord.xyz/shadow_tex_coord.w;
	//tex_coords = tex_coords * 0.5 +0.5;
	// read depth value from shadow map
	/*float depth = texture(shadow_map, tex_coords.xy).r;
	float inShadow = (depth < tex_coords.z) ? 0.0f : 1.0f;*/
	float shadow = chebyshevUpperBound(tex_coords.z);
	shadow = 1-shadow;

	vec4 textureColor = texture(colorTexture, fragmentUV).rgba;

	vec4 Ia = vec4(vec3(textureColor)*0.4,1); //ambient term
	
	vec3 n_surfaceNormal = normalize(surfaceNormal);
	vec3 lightDirection = lightPosition - surfacePosition;
	vec3 n_lightDirection = normalize(lightDirection);

	vec3 Id = textureColor.xyz * Ld * max(dot(n_lightDirection, n_surfaceNormal), 0.0) * lightPower*5.0;
		
	vec4 Id_shadow = vec4(Id, 0.0f)*shadow; //diffuse term
	fragColor = Ia+Id_shadow; 
	
	//fragColor=vec4((fragColor*(0.1+shadow*0.9)).xyz,1);

	//fragColor=vec4(vec3(shadow),1);

	//fragColor =vec4(vec3(inShadow),1); //Test
}